package com.persagy.apm.alarmservice.project.monitorindicatorconfig.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.dao.ProjectMonitorIndicatorConfigMapper;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.service.IProjectMonitorIndicatorConfigService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.model.*;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.model.dto.*;
import java.util.List;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 项目监测指标配置(ProjectMonitorIndicatorConfig) service层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:46:56
 */
@Service
public class ProjectMonitorIndicatorConfigServiceImpl extends ServiceImpl<ProjectMonitorIndicatorConfigMapper, ProjectMonitorIndicatorConfig> 
    implements IProjectMonitorIndicatorConfigService {
    
    /**
    * 创建项目监测指标配置
    * @return 项目监测指标配置主键
    * @author lixing
    * @version V1.0 2021-09-08 22:46:56
    */
    @Override
    public String createProjectMonitorIndicatorConfig(AddProjectMonitorIndicatorConfigDTO addProjectMonitorIndicatorConfigDTO) {
        ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig = ConvertProjectMonitorIndicatorConfigTool.INSTANCE.convertAddDto2Entity(addProjectMonitorIndicatorConfigDTO);
        // 设置默认值
        setDefaultValue(projectMonitorIndicatorConfig);
        save(projectMonitorIndicatorConfig);
        return projectMonitorIndicatorConfig.getId();
    }
    
    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param projectMonitorIndicatorConfig 项目监测指标配置实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig) {
        projectMonitorIndicatorConfig.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性
        
    }
    
    /**
    * 项目监测指标配置详情
    * @param id 主键
    * @return 部门do类
    * @author lixing
    * @version V1.0 2021-09-08 22:46:56
    */
    @Override
    public ProjectMonitorIndicatorConfig queryProjectMonitorIndicatorConfigDetail(String id) {
        ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig = getById(id);
        if (projectMonitorIndicatorConfig == null) {
            throw new IllegalArgumentException("查看ProjectMonitorIndicatorConfig详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return projectMonitorIndicatorConfig;
    }
    
    /**
    * 更新项目监测指标配置
    * @author lixing
    * @version V1.0 2021-09-08 22:46:56
    */
    @Override
    public void updateProjectMonitorIndicatorConfig(UpdateProjectMonitorIndicatorConfigDTO updateProjectMonitorIndicatorConfigDTO) {
        ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig = getById(updateProjectMonitorIndicatorConfigDTO.getId());
        projectMonitorIndicatorConfig = ConvertProjectMonitorIndicatorConfigTool.INSTANCE.convertUpdateDto2Entity(projectMonitorIndicatorConfig, updateProjectMonitorIndicatorConfigDTO);
        projectMonitorIndicatorConfig.setModifier(AppContext.getContext().getAccountId());
        updateById(projectMonitorIndicatorConfig);
    }
    
    /**
     * 校验项目监测指标配置是否可删除
     *
     * @param id 项目监测指标配置主键
     * @return 项目监测指标配置do类
     * @author lixing
     * @version V1.0 2021-09-08 22:46:56
     */
    public ProjectMonitorIndicatorConfig checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ProjectMonitorIndicatorConfig时发生异常，主键为空");
        }

        ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig = getById(id);

        if (projectMonitorIndicatorConfig == null) {
            throw new IllegalArgumentException("删除ProjectMonitorIndicatorConfig时发生异常，找不到要删除的数据，id:" + id);
        }

        return projectMonitorIndicatorConfig;
    }
    
    /**
    * 删除项目监测指标配置
    * @param id 主键
    * @author lixing
    * @version V1.0 2021-09-08 22:46:56
    */
    @Override
    public void deleteProjectMonitorIndicatorConfig(String id) {
        // 校验是否可删除
        ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig = checkDeletable(id);

        projectMonitorIndicatorConfig.setValid(ValidEnum.FALSE.getType());
        updateById(projectMonitorIndicatorConfig);
    }
    
    /**
    * 查询项目监测指标配置
    * @return List<ProjectMonitorIndicatorConfig>
    * @author lixing
    * @version V1.0 2021-09-08 22:46:56
    */
    @Override
    public List<ProjectMonitorIndicatorConfig> queryProjectMonitorIndicatorConfigList(QueryProjectMonitorIndicatorConfigDTO queryProjectMonitorIndicatorConfigDTO) {
        QueryWrapper<ProjectMonitorIndicatorConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectMonitorIndicatorConfig.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ProjectMonitorIndicatorConfig.PROP_CREATIONTIME);
        
        if (queryProjectMonitorIndicatorConfigDTO != null) {
                                
            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectMonitorIndicatorConfigDTO.getObjId())) {
                queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_OBJ_ID, queryProjectMonitorIndicatorConfigDTO.getObjId());
            }
                                        
            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectMonitorIndicatorConfigDTO.getMonitorIndicatorId())) {
                queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_MONITOR_INDICATOR_ID, queryProjectMonitorIndicatorConfigDTO.getMonitorIndicatorId());
            }
                                        
            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectMonitorIndicatorConfigDTO.getCaculateMethodId())) {
                queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_CACULATE_METHOD_ID, queryProjectMonitorIndicatorConfigDTO.getCaculateMethodId());
            }
                                        
            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectMonitorIndicatorConfigDTO.getProjectId())) {
                queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_PROJECT_ID, queryProjectMonitorIndicatorConfigDTO.getProjectId());
            }
                                            
        }
        
        return list(queryWrapper);
    }
    
    /**
    * 分页查询项目监测指标配置
    * @return IPage<ProjectMonitorIndicatorConfig>
    * @author lixing
    * @version V1.0 2021-09-08 22:46:56
    */
    @Override
    public IPage<ProjectMonitorIndicatorConfig> pageQueryProjectMonitorIndicatorConfig(PageQueryProjectMonitorIndicatorConfigDTO pageQueryProjectMonitorIndicatorConfigDTO) {
        QueryWrapper<ProjectMonitorIndicatorConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectMonitorIndicatorConfig.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ProjectMonitorIndicatorConfig> pageParam = new Page<>(pageQueryProjectMonitorIndicatorConfigDTO.getPage(), pageQueryProjectMonitorIndicatorConfigDTO.getSize(),true);
        // 排序信息
        if(CollectionUtils.isEmpty(pageQueryProjectMonitorIndicatorConfigDTO.getOrders())){
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ProjectMonitorIndicatorConfig.PROP_CREATIONTIME);
        }else {
            List<Sort> orders = pageQueryProjectMonitorIndicatorConfigDTO.getOrders();
            for(Sort sort: orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE,sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }
                
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectMonitorIndicatorConfigDTO.getObjId())) {
            queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_OBJ_ID, pageQueryProjectMonitorIndicatorConfigDTO.getObjId());
        }
                            
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectMonitorIndicatorConfigDTO.getMonitorIndicatorId())) {
            queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_MONITOR_INDICATOR_ID, pageQueryProjectMonitorIndicatorConfigDTO.getMonitorIndicatorId());
        }
                            
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectMonitorIndicatorConfigDTO.getCaculateMethodId())) {
            queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_CACULATE_METHOD_ID, pageQueryProjectMonitorIndicatorConfigDTO.getCaculateMethodId());
        }
                            
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectMonitorIndicatorConfigDTO.getProjectId())) {
            queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_PROJECT_ID, pageQueryProjectMonitorIndicatorConfigDTO.getProjectId());
        }
                                    
        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
